<?php

declare(strict_types=1);

namespace Apexl\Io\Interfaces;

interface HasCurrencyCastsInterface
{
    public function currencyCode(): string;

    public function currencyCodeField(): ?string;

    public function currencyMultiplier(string $field): int;
}