<?php

/** @noinspection PhpMissingFieldTypeInspection */
declare(strict_types=1);

namespace Apexl\Io\Logger\Dtos;

use Apexl\Io\Logger\Enums\LogLevel;
use Spatie\DataTransferObject\DataTransferObject;

class HandlerConfig extends DataTransferObject
{
    public string $destination;
    public LogLevel $level;
    public $params;
}
