<?php

declare(strict_types=1);

namespace Apexl\Io\Factories;

use Apexl\Io\Services\Mailer;
use DI\Container;

final readonly class MailerFactory
{
    public function __construct(private Container $container) {}

    public function make(): Mailer
    {
        return $this->container->make(Mailer::class);
    }
}