<?php

namespace Apexl\Io\Interfaces;

interface EntityInterface
{
    public function setField($field, $value);
    public function getData();
    public function getField($field);
    public function getEntityType();
    public function setOperator(EntityOperatorInterface $operator);
}
