<?php

declare(strict_types=1);

namespace Apexl\Io\Logger\Dtos\HandlerConfigParams;

use Apexl\Io\Logger\Dtos\HandlerConfigParams;

class FileHandlerConfigParams extends HandlerConfigParams
{
    public ?string $file;
    public int $maxFiles = 7;
}
