<?php

declare(strict_types=1);

namespace Apexl\Io\Dtos\Relationship\Decorators;

use Apexl\Io\Interfaces\QueryDecorator;
use Apexl\Io\Interfaces\RelationshipInterface;
use Apexl\Vault\Interfaces\QueryBuilderInterface;

final readonly class Identity implements QueryDecorator
{
    public function decorate(RelationshipInterface $relationship, QueryBuilderInterface $query): QueryBuilderInterface
    {
        return $query;
    }
}
