<?php


return [
    "environment" => \Apexl\Io\env('APP_ENVIRONMENT', 'production'),
    "name" => \Apexl\Io\env('APP_NAME', 'IO Application'),
    "email_address" => \Apexl\Io\env('APP_EMAIL_ADDRESS', 'no-reply@localhost'),
];