<?php

declare(strict_types=1);

namespace Apexl\Io\Cache\Collections;

use Apexl\Io\Cache\Dtos\RedisCacheDriverServerConfig;
use Arrayy\Collection\AbstractCollection;

class RedisCacheDriverServerConfigCollection extends AbstractCollection
{
    public function getType(): string
    {
        return RedisCacheDriverServerConfig::class;
    }
}
