<?php

declare(strict_types=1);

namespace Apexl\Io\Modules;

use Apexl\Io\Cache\Hooks\RegisterDefaultCacheStreams;
use Apexl\Io\Interfaces\ProvidesHooksInterface;
use Apexl\Io\Module;

final readonly class CacheModule extends Module implements ProvidesHooksInterface
{
    public function hooks(): array
    {
        return [
            RegisterDefaultCacheStreams::class,
        ];
    }
}