<?php

declare(strict_types=1);

namespace Apexl\Io\Cache\Interfaces;

use Stash\Interfaces\DriverInterface;

interface CacheDriverFactoryInterface
{
    public function make(array $options = []): DriverInterface;

    public function loadable(array $options): bool;
}